function hp=co(h,K);
%CO  Bi-orthogonal Complement filter
% CO(H) is the bi-orthogonal complement filter of low-pass linear-phase
% L-tap filter H. Its length is L if L is even (type A) and L-2 if L is
% odd (typeB). Its pass-band gain is 2/g where g is the pass-band gain
% of H.
%
% CO(H,K) forces K zeros at z==-1 in the complement filter. Its length
% will be L+2*(K-1) where L is the length of H. CO(H) is equivalent to
% CO(H,0) if L is odd, CO(H,1) if L is even. In all cases, L-K must be
% odd.

% For MATLAB 4.1
% Author: Olivier Rioul March 18 1994

% References:
% Olivier Rioul, Ondelettes regulieres: Application a la compression d'images
%     fixes, PhD Thesis, Telecom Paris, March 1993.

if min(size(h))~=1, error('input must be a vector'), end
if any(imag(h)), error('input must be real'), end
h=h(:)'; L=length(h);
if h~=h(L:-1:1),  error('input must be symetric'); end

if nargin<2, K=1-rem(L,2); end
if ~rem(L-K,2) % L-K even
 disp('warning: L-K is even, setting K to:');
 K=K+1
end

% Add K zeros at z==-1 in h
for k=1:K
 h=[h 0]+[0 h];
end
L=L+K;

% Linear system matrix in the case L odd
% Solution length is L-2
H=toeplitz(h',[h(1) zeros(1,L-1)]);
H=H(3:2:L,2:L-1);
H=H(:,1:(L-1)/2)+H(:,L-2:-1:(L-1)/2);
H(:,(L-1)/2)=H(:,(L-1)/2)/2;

% Inverting system
c=zeros((L-1)/2,1); c((L-1)/2)=1;
hp=(H\c)';
% Make hp symmetric
hp=[hp hp(length(hp)-1:-1:1)];

% Add K zeros at z==-1 in hp
for k=1:K
 hp=[hp 0]+[0 hp];
end
