function p=wlet(h,iter,dn);
%WLET plots wavelets, scaling functions and their derivatives
% WLET(H) plots scaling function or wavelet associated to filter H,
% if H is a low-pass or high-pass filter, respectively.
%
% WLET(H,n) forces the number of iterations to n
% By default, n is determined such that the curve has at least 200 points
%
% WLET(H,n,m) plots the m-th derivative of scaling function or wavelet
% (see above) using n iterations.

% For MATLAB 4.1
% Author: Olivier Rioul March 17 1994

Lc=200; % min # points in curve
Lh=length(h);
h=h(:)';
% first iteration j=1.
p=h;

% test if h is low-bas
hpi=sum(h(1:2:Lh))-sum(h(2:2:Lh));
if abs(sum(h))<abs(hpi)
 h=mirror(h);
end
% normalize
sh=sum(h)/2;
h=h/sh; p=p/sh;

% derivatives
if nargin<3, dn=0; end
for k=1:dn
 p=([p 0]-[0 p])*2;
 [h,r]=deconv(h*2,[1 1]);
 if any(r)~=0
   fprintf('wlet: approximation for derivative=%g\n',max(abs(r)));
 end
end


% default number of iterations
if nargin==1
 % length(pj): (2^j-1)(Lh-1)+1>=Lc
 iter=ceil(log((Lc-1)/(Lh-1)+1)/log(2));
 fprintf('%g iterations.\n',iter);
else
 if iter<1
  error('Not enough iterations');
 end
end

for j=2:iter
 L=length(p);
 pp=zeros(1,2*L-1);
 pp(1:2:2*L-1)=p; % over-sampling
 p=conv(pp,h); % low-pass filter
end

plot(linspace(0,Lh-1,length(p)),p);
% This is slightly cheating: normally curves goes from 0 to (L-1)/2^iter
axis([0 Lh-1 min(p) max(p)]);
